@@ -1,21 +1,44 @@
 from __future__ import annotations
 
-from typing import TYPE_CHECKING, Dict, List
+from types import MappingProxyType
+from typing import TYPE_CHECKING, Dict, List, Sequence
 
 import matplotlib.pyplot as plt
 import meshio
 import numpy as np
 import pyvista as pv
 import scipy
-import trimesh
 
-from . import mesh2d, mesh3d
 from .mpl.meshplot import _legend_with_triplot_fix
 from .region_markers import RegionMarker, RegionMarkerLike
 
 if TYPE_CHECKING:
     import open3d
 
+    from .mesh_container import MeshContainer
+
+
+class PruneZ0Mixin:
+    def prune_z_0(self):
+        """Drop third dimension (z) coordinates if present and all values are
+        equal to 0 (within tolerance).
+
+        For compatibility, sometimes a column with zeroes is added, for
+        example when exporting to gmsh2.2 format. This method drops that
+        column.
+        """
+        TOL = 1e-9
+
+        is_3_dimensional = self.points.shape[1] == 3
+        if not is_3_dimensional:
+            return
+
+        if not np.all(np.abs(self.points[:, 2]) < TOL):
+            raise ValueError(
+                'Coordinates in third dimension are not all equal to zero.')
+
+        self.points = self.points[:, 0:2]
+
 
 class BaseMesh:
     _cell_type: str = 'base'
@@ -23,9 +46,10 @@ class BaseMesh:
     def __init__(self,
                  points: np.ndarray,
                  cells: np.ndarray,
+                 fields: Dict[str, int] = None,
                  region_markers: List[RegionMarker] = None,
                  **cell_data):
-        """Summary.
+        """Base class for meshes.
 
         Parameters
         ----------
@@ -33,6 +57,8 @@ class BaseMesh:
             Array with points.
         cells : (i, j) np.ndarray[int]
             Index array describing the cells of the mesh.
+        fields : Dict[str, int]:
+            Mapping from field names to labels
         region_markers : List[RegionMarker], optional
             List of region markers used for assigning labels to regions.
             Defaults to an empty list.
@@ -40,13 +66,28 @@ class BaseMesh:
             Additional cell data. Argument must be a 1D numpy array
             matching the number of cells defined by `i`.
         """
-        self._label_key = 'labels'
+        default_key = 'physical'
+        if (not cell_data) or (default_key in cell_data):
+            self.default_key = default_key
+        else:
+            self.default_key = list(cell_data.keys())[0]
+
+        if not fields:
+            fields = {}
 
         self.points = points
         self.cells = cells
+        self.field_to_number = fields
         self.region_markers = [] if region_markers is None else region_markers
         self.cell_data = cell_data
 
+    @property
+    def number_to_field(self):
+        """Mapping from numbers to fields, proxy to `.field_to_number`."""
+        return MappingProxyType(
+            {v: k
+             for k, v in self.field_to_number.items()})
+
     def add_region_marker(self, region_marker: RegionMarkerLike):
         """Add marker to list of region markers.
 
@@ -63,6 +104,17 @@ class BaseMesh:
 
         self.region_markers.append(region_marker)
 
+    def add_region_markers(self, region_markers: Sequence[RegionMarkerLike]):
+        """Add marker to list of region markers.
+
+        Parameters
+        ----------
+        region_markers : List[RegionMarkerLike]
+            List of region markers passed to `.add_region_marker`.
+        """
+        for region_marker in region_markers:
+            self.add_region_marker(region_marker)
+
     def to_meshio(self) -> 'meshio.Mesh':
         """Return instance of `meshio.Mesh`."""
         cells = [
@@ -154,6 +206,25 @@ class BaseMesh:
         """Return centers of cells (mean of corner points)."""
         return self.points[self.cells].mean(axis=1)
 
+    def get_cell_data(self, key: str, default_value: float = 0) -> np.ndarray:
+        """Get cell data with optional default value.
+
+        Parameters
+        ----------
+        key : str
+            Key of the cell data to retrieve.
+        default_value : float, optional
+            Optional default value (if cell data does not exist)
+
+        Returns
+        -------
+        np.ndarray
+        """
+        try:
+            return self.cell_data[self.default_key]
+        except KeyError:
+            return np.ones(len(self.cells), dtype=int) * default_value
+
     @property
     def zero_labels(self):
         """Return zero labels as fallback."""
@@ -162,15 +233,15 @@ class BaseMesh:
     @property
     def labels(self):
         """Shortcut for cell labels."""
-        if not self.cell_data:
+        try:
+            return self.cell_data[self.default_key]
+        except KeyError:
             return self.zero_labels
 
-        return self.cell_data[self._label_key]
-
     @labels.setter
-    def labels(self, data: np.array):
+    def labels(self, data: np.ndarray):
         """Shortcut for setting cell labels."""
-        self.cell_data[self._label_key] = data
+        self.cell_data[self.default_key] = data
 
     @property
     def dimensions(self):
@@ -184,7 +255,6 @@ class LineMesh(BaseMesh):
     def plot_mpl(self,
                  ax: plt.Axes = None,
                  key: str = None,
-                 fields: Dict[int, str] = None,
                  **kwargs) -> plt.Axes:
         """Simple line mesh plot using `matplotlib`.
 
@@ -206,9 +276,6 @@ class LineMesh(BaseMesh):
         if not ax:
             fig, ax = plt.subplots()
 
-        if fields is None:
-            fields = {}
-
         if key is None:
             try:
                 key = tuple(self.cell_data.keys())[0]
@@ -221,7 +288,7 @@ class LineMesh(BaseMesh):
         for cell_data_val in np.unique(cell_data):
             vert_x, vert_y = self.points.T
 
-            name = fields.get(cell_data_val, cell_data_val)
+            name = self.number_to_field.get(cell_data_val, cell_data_val)
 
             lineplot(
                 ax,
@@ -232,6 +299,15 @@ class LineMesh(BaseMesh):
                 label=name,
             )
 
+        if self.region_markers:
+            mark_x, mark_y = np.array(
+                [m.coordinates for m in self.region_markers]).T
+            ax.scatter(mark_y,
+                       mark_x,
+                       marker='*',
+                       color='red',
+                       label='Region markers')
+
         ax.set_title(f'{self._cell_type} mesh')
         ax.axis('equal')
 
@@ -239,28 +315,22 @@ class LineMesh(BaseMesh):
 
         return ax
 
+    def triangulate(self, opts: str = 'pq30Aa100') -> MeshContainer:
+        """Triangulate mesh using `triangle`."""
+        from .mesh2d.helpers import simple_triangulate
+        points = self.points
+        segments = self.cells
+        regions = [[*m.coordinates, m.label, 0] for m in self.region_markers]
 
-class TriangleMesh(BaseMesh):
+        return simple_triangulate(points=points,
+                                  segments=segments,
+                                  regions=regions,
+                                  opts=opts)
+
+
+class TriangleMesh(BaseMesh, PruneZ0Mixin):
     _cell_type = 'triangle'
 
-    def prune_z_0(self):
-        """Drop third dimension (z) coordinates if present and all values are
-        equal to 0 (within tolerance).
-
-        For compatibility, sometimes a column with zeroes is added. This
-        method drops that column.
-        """
-        TOL = 1e-9
-
-        if self.dimensions < 3:
-            return
-
-        if not np.all(np.abs(self.points[:, 2]) < TOL):
-            raise ValueError(
-                'Coordinates in third dimension are not all equal to zero.')
-
-        self.points = self.points[:, 0:2]
-
     def plot(self, **kwargs):
         """Shortcut for `.plot_mpl`."""
         if self.dimensions == 2:
@@ -271,7 +341,6 @@ class TriangleMesh(BaseMesh):
     def plot_mpl(self,
                  ax: plt.Axes = None,
                  key: str = None,
-                 fields: Dict[int, str] = None,
                  **kwargs) -> plt.Axes:
         """Simple mesh plot using `matplotlib`.
 
@@ -294,9 +363,6 @@ class TriangleMesh(BaseMesh):
         if not ax:
             fig, ax = plt.subplots()
 
-        if fields is None:
-            fields = {}
-
         if key is None:
             try:
                 key = tuple(self.cell_data.keys())[0]
@@ -309,7 +375,7 @@ class TriangleMesh(BaseMesh):
         for cell_data_val in np.unique(cell_data):
             vert_x, vert_y = self.points.T
 
-            name = fields.get(cell_data_val, cell_data_val)
+            name = self.number_to_field.get(cell_data_val, cell_data_val)
 
             ax.triplot(vert_y,
                        vert_x,
@@ -324,8 +390,9 @@ class TriangleMesh(BaseMesh):
 
         return ax
 
-    def to_trimesh(self) -> 'trimesh.Trimesh':
+    def to_trimesh(self):
         """Return instance of `trimesh.Trimesh`."""
+        import trimesh
         return trimesh.Trimesh(vertices=self.points, faces=self.cells)
 
     def to_open3d(self) -> 'open3d.geometry.TriangleMesh':
@@ -359,7 +426,7 @@ class TriangleMesh(BaseMesh):
         return cls(points=points, cells=cells)
 
     @classmethod
-    def from_trimesh(cls, mesh: 'trimesh.Trimesh') -> TriangleMesh:
+    def from_trimesh(cls, mesh) -> TriangleMesh:
         """Return instance of `TriangleMesh` from trimesh."""
         return cls(points=mesh.vertices, cells=mesh.faces)
 
@@ -372,63 +439,6 @@ class TriangleMesh(BaseMesh):
 
         return mesh
 
-    def simplify(self, n_cells: int) -> TriangleMesh:
-        """Simplify triangular mesh using `open3d`.
-
-        Parameters
-        ----------
-        n_cells : int
-            Simplify mesh until this number of cells is reached.
-
-        Returns
-        -------
-        TriangleMesh
-        """
-        mesh_o3d = self.to_open3d()
-        simplified_o3d = mesh_o3d.simplify_quadric_decimation(int(n_cells))
-        return TriangleMesh.from_open3d(simplified_o3d)
-
-    def simplify_by_point_clustering(self,
-                                     voxel_size: float = 1.0) -> TriangleMesh:
-        """Simplify mesh geometry using point clustering.
-
-        Parameters
-        ----------
-        voxel_size : float, optional
-            Size of the target voxel within which points are grouped.
-
-        Returns
-        -------
-        TriangleMesh
-        """
-        import open3d
-        mesh_in = self.to_open3d()
-        mesh_smp = mesh_in.simplify_vertex_clustering(
-            voxel_size=voxel_size,
-            contraction=open3d.geometry.SimplificationContraction.Average)
-
-        return TriangleMesh.from_open3d(mesh_smp)
-
-    def smooth(self, iterations: int = 50) -> TriangleMesh:
-        """Smooth mesh using the Taubin filter in `trimesh`.
-
-        The advantage of the Taubin algorithm is that it avoids
-        shrinkage of the object.
-
-        Parameters
-        ----------
-        iterations : int, optional
-            Number of smoothing operations to apply
-
-        Returns
-        -------
-        TriangleMesh
-        """
-        mesh_tri = self.to_trimesh()
-        smoothed_tri = trimesh.smoothing.filter_taubin(mesh_tri,
-                                                       iterations=iterations)
-        return TriangleMesh.from_trimesh(smoothed_tri)
-
     def optimize(self,
                  *,
                  method='CVT (block-diagonal)',
@@ -463,12 +473,6 @@ class TriangleMesh(BaseMesh):
         )
         return TriangleMesh(points=points, cells=cells)
 
-    def subdivide(self, max_edge: int = 10, iters: int = 10) -> TriangleMesh:
-        """Subdivide triangles."""
-        from trimesh import remesh
-        points, cells = remesh.subdivide(self.points, self.cells)
-        return TriangleMesh(points=points, cells=cells)
-
     def tetrahedralize(self, **kwargs) -> 'TetraMesh':
         """Tetrahedralize a contour.
 
@@ -486,26 +490,6 @@ class TriangleMesh(BaseMesh):
         mesh = tetgen.tetrahedralize(self, **kwargs)
         return mesh
 
-    def pad(self, **kwargs) -> TriangleMesh:
-        """Pad a mesh.
-
-        Parameters
-        ----------
-        **kwargs
-            Keyword arguments passed to `nanomesh.mesh2d.helpers.pad`
-        """
-        return mesh2d.pad(self, **kwargs)
-
-    def pad3d(self, **kwargs) -> TriangleMesh:
-        """Pad a 3d mesh.
-
-        Parameters
-        ----------
-        **kwargs
-            Keyword arguments passed to `nanomesh.mesh3d.helpers.pad`
-        """
-        return mesh3d.pad(self, **kwargs)
-
 
 class TetraMesh(BaseMesh):
     _cell_type = 'tetra'
@@ -518,7 +502,7 @@ class TetraMesh(BaseMesh):
             tetras=open3d.utility.Vector4iVector(self.cells))
 
     @classmethod
-    def from_open3d(cls, mesh):
+    def from_open3d(cls, mesh) -> TetraMesh:
         """Return instance of `TetraMesh` from open3d."""
         points = np.asarray(mesh.vertices)
         cells = np.asarray(mesh.tetras)
