@@ -5,9 +5,12 @@ from typing import TYPE_CHECKING
 import matplotlib.pyplot as plt
 import numpy as np
 import triangle as tr
+from scipy.spatial.distance import cdist
+
+from ..region_markers import RegionMarker
 
 if TYPE_CHECKING:
-    from nanomesh.mesh import TriangleMesh
+    from nanomesh.mesh import LineMesh, TriangleMesh
     from nanomesh.mesh_container import MeshContainer
 
 
@@ -83,107 +86,112 @@ def simple_triangulate(points: np.ndarray,
     return mesh
 
 
-def pad(mesh: TriangleMesh,
-        *,
-        side: str,
-        width: int,
-        opts: str = '',
-        label: int = None) -> TriangleMesh:
-    """Pad a triangle mesh.
+def pad(
+    mesh: LineMesh,
+    *,
+    side: str,
+    width: int,
+    label: int = None,
+    name: str = None,
+) -> LineMesh:
+    """Pad a triangle mesh (2D).
 
     Parameters
     ----------
-    mesh : TriangleMesh
+    mesh : LineMesh
         The mesh to pad.
     side : str
         Side to pad, must be one of `left`, `right`, `top`, `bottom`.
     width : int
         Width of the padded area.
-    opts : str, optional
-        Optional arguments passed to `triangle.triangulate`.
     label : int, optional
         The label to assign to the padded area. If not defined, generates the
         next unique label based on the existing ones.
+    name : str, optional
+        Name of the added region. Note that in case of conflicts, the `label`
+        takes presedence over the `name`.
 
     Returns
     -------
-    new_mesh : TriangleMesh
-        Padded triangle mesh.
+    new_mesh : LineMesh
+        Padded line mesh.
 
     Raises
     ------
     ValueError
         When the value of `side` is invalid.
     """
-    from nanomesh.mesh import TriangleMesh
+    labels = [m.label for m in mesh.region_markers]
+    names = [m.name for m in mesh.region_markers]
+
+    if (label in labels) and (name is None):
+        name = [m.name for m in mesh.region_markers if m.label == label][0]
+
+    if name and (name in names) and (label is None):
+        label = [m.label for m in mesh.region_markers if m.name == name][0]
 
     if label is None:
-        label = mesh.labels.max() + 1
+        label = max(max(labels) + 1, 2)
 
     if width == 0:
         return mesh
 
-    top_edge, right_edge = mesh.points.max(axis=0)
-    bottom_edge, left_edge = mesh.points.min(axis=0)
+    bottom_edge, right_edge = mesh.points.max(axis=0)
+    top_edge, left_edge = mesh.points.min(axis=0)
 
-    if side == 'bottom':
-        is_edge = mesh.points[:, 0] == bottom_edge
-        corners = np.array([[bottom_edge - width, right_edge],
-                            [bottom_edge - width, left_edge]])
+    if side == 'top':
+        corners = np.array([
+            [top_edge, right_edge],
+            [top_edge - width, right_edge],
+            [top_edge - width, left_edge],
+            [top_edge, left_edge],
+        ])
     elif side == 'left':
-        is_edge = mesh.points[:, 1] == left_edge
-        corners = np.array([[bottom_edge, left_edge - width],
-                            [top_edge, left_edge - width]])
-    elif side == 'top':
-        is_edge = mesh.points[:, 0] == top_edge
-        corners = np.array([[top_edge + width, right_edge],
-                            [top_edge + width, left_edge]])
+        corners = np.array([
+            [top_edge, left_edge],
+            [top_edge, left_edge - width],
+            [bottom_edge, left_edge - width],
+            [bottom_edge, left_edge],
+        ])
+    elif side == 'bottom':
+        corners = np.array([
+            [bottom_edge, right_edge],
+            [bottom_edge + width, right_edge],
+            [bottom_edge + width, left_edge],
+            [bottom_edge, left_edge],
+        ])
     elif side == 'right':
-        is_edge = mesh.points[:, 1] == right_edge
-        corners = np.array([[bottom_edge, right_edge + width],
-                            [top_edge, right_edge + width]])
+        corners = np.array([
+            [top_edge, right_edge],
+            [top_edge, right_edge + width],
+            [bottom_edge, right_edge + width],
+            [bottom_edge, right_edge],
+        ])
     else:
         raise ValueError('Side must be one of `right`, `left`, `bottom`'
                          f'`top`. Got {side=}')
 
-    edge_coords = mesh.points[is_edge]
+    all_points = mesh.points
 
-    coords = np.vstack([edge_coords, corners])
+    corner_idx = np.argwhere(cdist(corners, all_points) == 0)
 
-    pad_mesh = simple_triangulate(points=coords, opts=opts)
+    if len(corner_idx) < len(corners):
+        # Add missing corners and add them where necessary
+        missing_corners = np.delete(corners, corner_idx[:, 0], axis=0)
+        all_points = np.vstack([all_points, missing_corners])
+        corner_idx = np.argwhere(cdist(corners, all_points) == 0)
 
-    mesh_edge_index = np.argwhere(is_edge).flatten()
-    pad_edge_index = np.arange(len(mesh_edge_index))
-    edge_mapping = np.vstack([pad_edge_index, mesh_edge_index])
+    R = corner_idx[:, 1].tolist()
+    additional_segments = list(zip(R, R[1:] + R[:1]))
+    cells = np.vstack([mesh.cells, additional_segments])
 
-    n_verts = len(mesh.points)
-    n_edge_verts = len(edge_coords)
-    n_pad_verts = len(pad_mesh.points) - n_edge_verts
+    center = corners.mean(axis=0)
+    region_markers = mesh.region_markers + [RegionMarker(label, center, name)]
 
-    mesh_index = np.arange(n_verts, n_verts + n_pad_verts)
-    pad_index = np.arange(n_edge_verts, n_edge_verts + n_pad_verts)
-    pad_mapping = np.vstack([pad_index, mesh_index])
-
-    # mapping for the cell indices cells in `pad_mesh` to the source mesh.
-    mapping = np.hstack([edge_mapping, pad_mapping])
-
-    cells = pad_mesh.cells_dict['triangle'].copy()
-    shape = cells.shape
-    pad_cells = cells.ravel()
-
-    mask = np.in1d(pad_cells, mapping[0, :])
-    pad_cells[mask] = mapping[1,
-                              np.searchsorted(mapping[0, :], pad_cells[mask])]
-    pad_cells = pad_cells.reshape(shape)
-
-    pad_verts = pad_mesh.points[n_edge_verts:]
-    pad_labels = np.ones(len(pad_cells)) * label
-
-    # append values to source mesh
-    points = np.vstack([mesh.points, pad_verts])
-    cells = np.vstack([mesh.cells, pad_cells])
-    labels = np.hstack([mesh.labels, pad_labels])
-
-    new_mesh = TriangleMesh(points=points, cells=cells, labels=labels)
+    new_mesh = mesh.__class__(
+        points=all_points,
+        cells=cells,
+        region_markers=region_markers,
+    )
 
     return new_mesh
