@@ -8,7 +8,7 @@ from typing import Dict, List
 import meshio
 import numpy as np
 
-from .mesh import BaseMesh
+from .mesh import BaseMesh, PruneZ0Mixin
 
 
 class CellType(Enum):
@@ -18,7 +18,7 @@ class CellType(Enum):
     TETRA = 3
 
 
-class MeshContainer(meshio.Mesh):
+class MeshContainer(meshio.Mesh, PruneZ0Mixin):
     @property
     def number_to_field(self):
         """Mapping from numbers to fields, proxy to `.field_data`."""
@@ -28,7 +28,9 @@ class MeshContainer(meshio.Mesh):
             dim_name = CellType(dimension).name.lower()
             number_to_field[dim_name][number] = field
 
-        return MappingProxyType(dict(number_to_field))
+        return MappingProxyType(
+            {k: MappingProxyType(v)
+             for k, v in number_to_field.items()})
 
     @property
     def field_to_number(self):
@@ -39,7 +41,9 @@ class MeshContainer(meshio.Mesh):
             dim_name = CellType(dimension).name.lower()
             field_to_number[dim_name][field] = number
 
-        return MappingProxyType(dict(field_to_number))
+        return MappingProxyType(
+            {k: MappingProxyType(v)
+             for k, v in field_to_number.items()})
 
     def set_field_data(self, cell_type: str, field_data: Dict[int, str]):
         """Update the values in `.field_data`.
@@ -47,14 +51,14 @@ class MeshContainer(meshio.Mesh):
         Parameters
         ----------
         cell_type : str
-            Cell type to update the values ofr.
+            Cell type to update the values for.
         field_data : dict
             Dictionary with key-to-number mapping, i.e.
             `field_data={0: 'green', 1: 'blue', 2: 'red'}`
             maps `0` to `green`, etc.
         """
         try:
-            input_field_data = dict(self.number_to_field)[cell_type]
+            input_field_data = dict(self.number_to_field[cell_type])
         except KeyError:
             input_field_data = {}
 
@@ -144,7 +148,12 @@ class MeshContainer(meshio.Mesh):
 
         cell_data = self.get_all_cell_data(cell_type)
 
-        return BaseMesh.create(cells=cells, points=points, **cell_data)
+        fields = self.field_to_number.get(cell_type, None)
+
+        return BaseMesh.create(cells=cells,
+                               points=points,
+                               fields=fields,
+                               **cell_data)
 
     def get_all_cell_data(self, cell_type: str = None) -> dict:
         """Get all cell data for given `cell_type`.
@@ -270,7 +279,7 @@ class MeshContainer(meshio.Mesh):
             cells['line'] = triangle_dict['edges']
             # Order must match order of cell_data
             cell_data['physical'] = [
-                np.zeros(len(cells['triangle'])),
+                triangle_dict['triangle_attributes'].squeeze(),
                 triangle_dict['edge_markers'].squeeze(),
             ]
         except KeyError:
@@ -283,6 +292,8 @@ class MeshContainer(meshio.Mesh):
             point_data=point_data,
         )
 
+        mesh.triangle_dict = triangle_dict
+
         return mesh
 
     @classmethod
@@ -295,7 +306,6 @@ class MeshContainer(meshio.Mesh):
         """
         from meshio import read
         mesh = read(*args, **kwargs)
-        mesh.prune_z_0()
 
         cell_data = {}
         for key, value in mesh.cell_data.items():
@@ -311,7 +321,7 @@ class MeshContainer(meshio.Mesh):
 
             point_data[key] = value
 
-        return cls(
+        ret = cls(
             mesh.points,
             mesh.cells,
             point_data=point_data,
@@ -322,6 +332,8 @@ class MeshContainer(meshio.Mesh):
             gmsh_periodic=mesh.gmsh_periodic,
             info=mesh.info,
         )
+        ret.prune_z_0()
+        return ret
 
     def write(self, filename, file_format: str = None, **kwargs):
         """Thin wrapper of `meshio.write` to avoid altering class.
